dxf="parts.dxf";
deck_thickness=9;
beam_size=dxf_dim(file=dxf, layer="dimensions", name="beam_size");
robot_width=dxf_dim(file=dxf, layer="dimensions", name="robot_width");
robot_depth=dxf_dim(file=dxf, layer="dimensions", name="robot_depth");
robot_half_width=robot_width/2;
deck_dist=300;
wheel_void_width=dxf_dim(file=dxf, layer="dimensions", name="wheel_void_width");
wheel_width=24;
wheel_axis_y=50;
wheel_axis_z=20;

module upper_deck() {
	linear_extrude(height=deck_thickness, $fn=100)
		import(file=dxf, layer="upper_deck");
}

module lower_deck() {
	linear_extrude(height=deck_thickness, $fn=100)
		import(file=dxf, layer="lower_deck");
}

module beam(len) {
	module beam_hole_(d) {
		d_hole = 5;
		center_dist = d - d_hole;
		hull() {
			translate([-center_dist/2,0,0])
				cylinder(r=d_hole/2, h=beam_size+1, center=true);
			translate([center_dist/2,0,0])
				cylinder(r=d_hole/2, h=beam_size+1, center=true);
		}
	}

	module beam_hole_1() {
		beam_hole_(10);
	}

	module beam_hole_2() {
		union() {
			beam_hole_(30);
			cylinder(r=5, h=beam_size+1, center=true) ;
		}
	}

	h1_dist = 25;
	h2_dist = 2*h1_dist;
	color("silver")
	difference() {
		translate([-beam_size/2, -beam_size/2,0]) {
			difference(){
				cube([beam_size, beam_size, len]);
				translate([1,1,-1])
					cube([beam_size-2, beam_size-2, len+2]);
			}
		}
		rotate([90,0,0]) {
			for (y=[h1_dist/2:h1_dist:len]) {
				translate([0, y, 0]) beam_hole_1();
			}
			for (y=[h1_dist:h2_dist:len]) {
				translate([0,y,0]) rotate([0,90,0]) rotate([0,0,90]) beam_hole_2();
			}
		}
	}
}

module wheel() {
	rotate([0,90,0]){
		color("indigo")
			difference() {
				linear_extrude(height=wheel_width, center=true, $fn=60, convexity=10)
					import(file=dxf, layer="wheel_hub");
				rotate_extrude(convexity=10, $fn=60)
					import(file=dxf, layer="wheel_spokes_substract");
			}
		color("dimgray")
			rotate_extrude($fn=60)
				import(file=dxf, layer="wheel_tyre");
	}
}

module ball_caster() {
	color("grey")
	translate([0,0,-10]) {
		translate([0,0,10]) difference() {
			rotate_extrude(convexity=10) translate([10,0,0]) circle(r=10);
			translate([0,0,20]) cube([40,40,40], center=true);
		}
		sphere(r=10);
	}
}

module gear() {
	color("ghostwhite")
	cylinder(r1=4,r2=12,h=8, center=true);
}

module motor() {
	rotate([-90,0,0]) {
		color("lightgrey")
		cylinder(h=25, r=19, $fn=60);
		color("grey")
		translate([0,0,25]) cylinder(h=35, r=20, $fn=60);
		color("lightgrey")
		translate([0,0,60]) cylinder(h=15, r=18, $fn=60);
		// axle
		color("dimgrey")
		translate([0,0,-24]) cylinder(h=30, r=3, $fn=60);
		translate([0,0,-19]) gear();
	}
}

module odometer() {
	rotate([90,60,90]) {
		color("dimgray")
		linear_extrude(height=8, $fn=60) 
			import(file=dxf, layer="odometer_arm");

		color("red") translate([0,0,4]) cylinder(r=4, h=8, center=true, $fn=30);
		color("dimgray") translate([0,0,-8]) cylinder(r=2, h=32, center=true, $fn=20);

		translate([30,0,10]) rotate([0,0,90]) {
			color("black") rotate_extrude() translate([13,0,0]) circle(r=1.5);
			color("silver") cylinder(r=12,h=3, center=true, $fn=60);
			color("green") translate([0,0,-11]) cylinder(r=6, h=12);
			color("dimgray") cylinder(r=2, h=4, center=true, $fn=20);
			color("orange") rotate([0,0,60]) translate([-2,0,-10]) cube([3,8,5], center=true);
		}
	}
}

module propulsion_group() {
	translate([-33, 28, 0]) {
		motor();
		color("grey") {
			translate([-20,-28,0]) cube([15,45,40], center=true);
			translate([-5,-4,0]) cube([45,4,40], center=true);
		}
		// axle
		color("dimgray")
			translate([-28,-28,0]) rotate([0,90,0]) cylinder(r=4, h=75, $fn=30);
	}
	// ball bearing
	color("red") 
		rotate([0,90,0]) cylinder(r=11, h=wheel_width, center=true);
	// wheel and attached gear
	translate([-25,0,0]) rotate([0,90,0]) {
		gear();
		// connector
		color("dimgray") translate([0,0,4]) cylinder(r=12,h=10, $fn=60);
	}
	wheel();

	translate([-37,-15,-10])
		odometer();
}

module battery_pack() {
	color("beige")
	translate([0,0,135]) 
	minkowski() {
		sphere(r=5);
		cube([180,15,230], center=true);
	}
}

module untrimmed_robot() {
		beam_center_x = robot_half_width - wheel_void_width - beam_size / 2;
		l = 2 * (beam_center_x - beam_size/2);
		for (y=[0,170]) {
			for (x = [-1, +1]) {
				translate([beam_center_x * x, y + beam_size/2, deck_thickness])
					beam(deck_dist-deck_thickness);
			}

			translate([0, y, deck_dist-beam_size/2])
				rotate(90, [0,1,0])
					translate([0, beam_size/2, -l/2])
						beam(l);
		}

		d = robot_width/2 - wheel_width/2;
		translate([d, 50, 20+deck_thickness])
			propulsion_group();
		translate([-d, 50, 20+deck_thickness])
			mirror([1,0,0])
			propulsion_group();
		translate([0,robot_depth-30,0]) ball_caster();
		
		// desks added at end to avoid graphical artifacts
		translate([0, 0, deck_dist])
			upper_deck();

		lower_deck();

		translate([0, 210, 0]) battery_pack();
}

module robot() {
	translate([0, -wheel_axis_y, wheel_axis_z]) 
		untrimmed_robot();
}

robot();
